package fs

import (
	"errors"
	"fmt"
	"os"
)

// Unit represents the size multiplier
type Unit int64

const (
	// GiB is 1024 * 1024 * 1024 bytes
	GiB Unit = 1024
	// GB is 1000 * 1000 * 1000 bytes
	GB Unit = 1000
)

var (
	// ErrInvalidSizeMultiplier is returned when using an unsupported Unit
	ErrInvalidSizeMultiplier = errors.New("unsupported sizing unit used")
)

// Truncate creates an empty file of the specified size
func Truncate(path string, mode os.FileMode, size int64, u Unit) error {
	file, err := os.Create(path)
	if err != nil {
		return err
	}
	defer func() {
		file.Close()
		if err != nil {
			os.Remove(path)
		}
	}()

	switch u {
	case GiB:
		size = size * 1024 * 1024 * 1024
	case GB:
		// The image size will be reduced to fit commercial drives that are
		// smaller than what they claim, 975 comes from 97.5% of the total size
		size = size * 1000 * 1000 * 975
	default:
		return ErrInvalidSizeMultiplier
	}

	if err := file.Truncate(size); err != nil {
		return fmt.Errorf("error creating %s of size %d", path, size)
	}

	return file.Chmod(mode)
}
