package grub

import (
	"errors"
	"fmt"
	"io"
	"os"
	"os/exec"
	"path/filepath"
	"runtime"

	"launchpad.net/snappy/tools/fs"
)

// Architecture is used to determine the architecture used by grub
type Architecture string

const (
	// ArchAmd64 represents the amd64 architecture
	ArchAmd64 Architecture = "amd64"
	// Arch386 represents the 386 architecture, also known as i386
	Arch386 Architecture = "386"
	// ArchArm represents the arm hard float architecture, also known as armhf
	ArchArm Architecture = "arm"
)

const grubStubContent = `set prefix=($root)'/'
configfile $prefix/grub.cfg
`

// InstallCoreImage creates a grub core.img for the device
func InstallCoreImage(device string) error {
	arch := Architecture(runtime.GOARCH)
	if out, err := exec.Command("grub-install", "--target="+grubTarget(arch), "--grub-setup=/bin/true", "--recheck", device).CombinedOutput(); err != nil {
		return fmt.Errorf("%s", out)
	}

	return nil
}

// ChrootInstall installs grub to rootPath
func ChrootInstall(device, rootPath, bootPath string, arch Architecture) error {
	for _, dev := range []string{"dev", "proc", "sys", "tmp"} {
		src := filepath.Join("/", dev)
		dst := filepath.Join(rootPath, dev)
		if err := fs.BindMount(src, dst); err != nil {
			return err
		}
		defer fs.Unmount(dst)
	}

	firmwarePath := filepath.Join(rootPath, "sys", "firmware")
	if err := fs.BindMount(filepath.Join(rootPath, "mnt"), firmwarePath); err != nil {
		return err
	}
	defer fs.Unmount(firmwarePath)

	outputPath, err := filepath.Abs(device)
	if err != nil {
		return errors.New("cannot determined absolute path for output image")
	}

	// create root_dev in tmp to workaround ro filesystems
	rootDevPath := filepath.Join(rootPath, "tmp", "root_dev")

	f, err := os.Create(rootDevPath)
	if err != nil {
		return err
	}
	f.Close()
	defer os.Remove(rootDevPath)

	if err := fs.BindMount(outputPath, rootDevPath); err != nil {
		return err
	}
	defer fs.Unmount(rootDevPath)

	efiDir := filepath.Join(rootPath, "boot", "efi")
	if err := os.MkdirAll(efiDir, 0755); err != nil {
		return fmt.Errorf("unable to create %s dir: %s", efiDir, err)
	}

	if err := fs.BindMount(bootPath, efiDir); err != nil {
		return err
	}
	defer fs.Unmount(efiDir)

	// create efi layout
	efiGrubDir := filepath.Join(rootPath, "boot", "efi", "EFI", "ubuntu", "grub")
	if err := os.MkdirAll(efiGrubDir, 0755); err != nil {
		return fmt.Errorf("unable to create %s dir: %s", efiGrubDir, err)
	}

	bootGrubDir := filepath.Join(rootPath, "boot", "grub")
	if err := fs.BindMount(efiGrubDir, bootGrubDir); err != nil {
		return err
	}
	defer fs.Unmount(bootGrubDir)

	if arch == ArchAmd64 || arch == Arch386 {
		// install grub BIOS support
		if out, err := exec.Command("chroot", rootPath, "grub-install", "/tmp/root_dev").CombinedOutput(); err != nil {
			return fmt.Errorf("unable to install grub (BIOS): %s", out)
		}
	}

	// install grub EFI
	if out, err := exec.Command("chroot", rootPath, "grub-install", fmt.Sprint("--target="+grubEfiTarget(arch)), "--no-nvram", "--removable", "--efi-directory=/boot/efi").CombinedOutput(); err != nil {
		return fmt.Errorf("unable to install grub (EFI): %s", out)
	}

	// tell our EFI grub where to find its full config
	efiBootDir := filepath.Join(rootPath, "boot", "efi", "EFI", "BOOT")
	grubStub, err := os.Create(filepath.Join(efiBootDir, "grub.cfg"))
	if err != nil {
		return fmt.Errorf("unable to create %s file: %s", grubStub.Name(), err)
	}
	defer grubStub.Close()
	if _, err := io.WriteString(grubStub, grubStubContent); err != nil {
		return err
	}

	return nil
}

func grubEfiTarget(arch Architecture) string {
	switch arch {
	case ArchArm:
		return "arm-efi"
	case ArchAmd64:
		return "x86_64-efi"
	case Arch386:
		return "i386-efi"
	default:
		panic("unsupported architecture for GRUB on EFI")
	}
}

func grubTarget(arch Architecture) string {
	switch arch {
	case ArchArm:
		return "arm-uboot"
	case ArchAmd64:
		return "i386-pc"
	case Arch386:
		return "i386-pc"
	default:
		panic("unsupported architecture for GRUB (BIOS)")
	}
}
