package mapper

import (
	"bufio"
	"fmt"
	"os/exec"
	"path/filepath"
	"strings"
)

// DevMap is a struct that holds device node mappings.
type DevMap struct {
	Nodes []string
	img   string
}

// New creates a new mapper instance.
func New(img string) *DevMap {
	return &DevMap{img: img}
}

// Map creates a device map for img.
func (s *DevMap) Map() error {
	kpartxCmd := exec.Command("kpartx", "-avs", s.img)
	stdout, err := kpartxCmd.StdoutPipe()
	if err != nil {
		return err
	}

	if err := kpartxCmd.Start(); err != nil {
		return err
	}

	scanner := bufio.NewScanner(stdout)
	for scanner.Scan() {
		fields := strings.Fields(scanner.Text())

		if len(fields) > 2 {
			s.Nodes = append(s.Nodes, filepath.Join("/dev", "mapper", fields[2]))
		} else {
			return fmt.Errorf("issues while determining drive mappings (%q)", fields)
		}
	}
	if err := scanner.Err(); err != nil {
		return err
	}

	return kpartxCmd.Wait()
}

// Unmap removes the device mapping for img while clearing loops.
func (s *DevMap) Unmap() error {
	for _, devNode := range s.Nodes {
		dmsetupCmd := []string{"dmsetup", "clear", devNode}
		if out, err := exec.Command(dmsetupCmd[0], dmsetupCmd[1:]...).CombinedOutput(); err != nil {
			return &ErrExec{command: dmsetupCmd, output: out}
		}
	}

	kpartxCmd := []string{"kpartx", "-ds", s.img}
	if out, err := exec.Command(kpartxCmd[0], kpartxCmd[1:]...).CombinedOutput(); err != nil {
		return &ErrExec{command: kpartxCmd, output: out}
	}

	return nil
}

// ErrExec is an error from an external command
type ErrExec struct {
	output  []byte
	command []string
}

func (e ErrExec) Error() string {
	return fmt.Sprintf("error while executing external command %s: %s", strings.Join(e.command, " "), e.output)
}
