package parted

import (
	"errors"
	"fmt"
	"os"
	"os/exec"
	"strings"
)

// This is used for debugging
var debugPrint bool

// PartitionLabel is the label a partition has
type PartitionLabel string

// MklabelType represents the partition type (dos/gpt)
type MklabelType string

// FsType is the filesystem type to use for the partition
type FsType string

const (
	//MkLabelGpt represents gpt partitioning
	MkLabelGpt MklabelType = "gpt"
	//MkLabelMsdos represents dos/legacy style partitioning
	MkLabelMsdos MklabelType = "msdos"
)

const (
	// FsFat32 specifies the partition formatting to be of fat32/vfat.
	FsFat32 FsType = "fat32"
	// FsExt4 specifies the partition formatting to be of ext4.
	FsExt4 FsType = "ext4"
	// FsNone specifies an unformatted partition.
	FsNone FsType = ""
)

// ErrUnsupportedPartitioning is returned when an unsupported partitioning
// format is chosen.
var ErrUnsupportedPartitioning = errors.New("unsupported partitioning")

// ErrExec is an error from an external command
type ErrExec struct {
	output  []byte
	command []string
}

func (e ErrExec) Error() string {
	return fmt.Sprintf("error while executing external command %s: %s", strings.Join(e.command, " "), e.output)
}

func execute(cmd []string) error {
	if out, err := exec.Command(cmd[0], cmd[1:]...).CombinedOutput(); err != nil {
		return &ErrExec{command: cmd, output: out}
	}

	return nil
}

func init() {
	if e := os.Getenv("PARTS"); e != "" {
		debugPrint = true
	}
}
