// -*- Mode: Go; indent-tabs-mode: t -*-

/*
 * Copyright (C) 2014-2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package parted

import (
	"encoding/csv"
	"errors"
	"fmt"
	"io"
	"os"
	"os/exec"
	"strconv"
	"strings"
)

// Parted represents a working structure that interacts with the parted command.
type Parted struct {
	mklabel       MklabelType
	Parts         []Partition
	bootPartition int
	biosGrub      int
	device        string
	fresh         bool
}

// New creates a new parted instance
func New(device string, mklabel MklabelType) (*Parted, error) {
	if mklabel != MkLabelGpt && mklabel != MkLabelMsdos {
		return nil, ErrUnsupportedPartitioning
	}

	return &Parted{
		device:  device,
		mklabel: mklabel,
		fresh:   true,
	}, nil
}

// NewExisting creates a new parted instance from an existing partitioning. if
// device is "" the existing partition will be used.
func NewExisting(device string) (*Parted, error) {
	p := Parted{
		device: device,
	}

	if err := p.loadPartitions(); err != nil {
		return nil, err
	}

	return &p, nil
}

func (p *Parted) loadPartitions() error {
	partedCmd := exec.Command("parted", "-m", p.device, "unit", "s", "print")

	stdout, err := partedCmd.StdoutPipe()
	if err != nil {
		return err
	}

	if err := partedCmd.Start(); err != nil {
		return err
	}

	reader := csv.NewReader(stdout)
	reader.Comma = ':'
	reader.FieldsPerRecord = 7

	for {
		record, err := reader.Read()
		if err == io.EOF {
			break
		} else if parseErr, ok := err.(*csv.ParseError); ok && parseErr.Err == csv.ErrFieldCount {
			fmt.Println("skipping record:", strings.Join(record, ":"))
			continue
		} else if err != nil {
			return err
		}

		if record[0] == p.device {
			switch record[5] {
			case "gpt":
				p.mklabel = MkLabelGpt
			case "msdos":
				p.mklabel = MkLabelMsdos
			}
		} else {
			var part Partition
			var err error

			if strings.HasPrefix(p.device, "/dev") {
				part.Node = fmt.Sprintf("%s%s", p.device, record[0])
			}

			part.begin, err = strconv.ParseInt(record[1][:len(record[1])-1], 0, 64)
			if err != nil {
				return err
			}

			part.end, err = strconv.ParseInt(record[2][:len(record[2])-1], 0, 64)
			if err != nil {
				return err
			}

			switch record[4] {
			case "fat32":
				part.fs = FsFat32
			case "ext4":
				part.fs = FsExt4
			default:
				part.fs = FsNone
			}

			part.Label = PartitionLabel(record[5])

			p.Parts = append(p.Parts, part)
		}
	}

	return partedCmd.Wait()
}

// Add adds a partition to the image or disk.
func (p *Parted) Add(label PartitionLabel, fs FsType, size int64) {
	var begin int64

	if len(p.Parts) == 0 {
		begin = 8192
	} else {
		begin = p.Parts[len(p.Parts)-1].end + 1
	}

	end := size
	if size != -1 {
		end = mib2Blocks(size) + begin - 1
	}

	part := Partition{
		begin: begin,
		end:   end,
		Label: label,
		fs:    fs,
	}

	p.Parts = append(p.Parts, part)
}

// Apply runs through the added parts and actually creates the image accordingly
// to the configuration.
func (p *Parted) Apply() error {
	if p.mklabel == MkLabelMsdos && len(p.Parts) > 4 {
		panic("invalid amount of partitions for msdos")
	}

	partedCmd := exec.Command("parted", p.device)
	stdin, err := partedCmd.StdinPipe()
	if err != nil {
		return err
	}

	stdout, err := partedCmd.StdoutPipe()
	if err != nil {
		return err
	}

	stderr, err := partedCmd.StderrPipe()
	if err != nil {
		return err
	}

	if err := partedCmd.Start(); err != nil {
		return err
	}

	if p.fresh {
		stdin.Write([]byte(fmt.Sprintf("mklabel %s\n", p.mklabel)))
	}

	for i, part := range p.Parts {
		var fmtStr string
		if part.end != -1 {
			fmtStr = "mkpart %s %s %ds %ds\n"
		} else {
			fmtStr = "mkpart %s %s %ds %dM\n"
		}

		// already exists so don't create
		if part.Node != "" {
			continue
		}

		if strings.HasPrefix(p.device, "/dev/") {
			p.Parts[i].Node = fmt.Sprintf("%s%d", p.device, i+1)
		}

		switch p.mklabel {
		case MkLabelGpt:
			stdin.Write([]byte(fmt.Sprintf(fmtStr, part.Label, part.fs, part.begin, part.end)))
		case MkLabelMsdos:
			stdin.Write([]byte(fmt.Sprintf(fmtStr, "primary", part.fs, part.begin, part.end)))
		}
	}

	if p.bootPartition != 0 {
		stdin.Write([]byte(fmt.Sprintf("set %d boot on\n", p.bootPartition)))
	}

	if p.biosGrub != 0 {
		stdin.Write([]byte(fmt.Sprintf("set %d bios_grub on\n", p.biosGrub)))
	}

	stdin.Write([]byte("quit\n"))

	if debugPrint {
		go io.Copy(os.Stdout, stdout)
		go io.Copy(os.Stderr, stderr)
	}

	if err := partedCmd.Wait(); err != nil {
		return errors.New("issues while partitioning")
	}

	return nil
}

// SetBiosGrub tells Parted that a partition number is to be set as bios.
func (p *Parted) SetBiosGrub(partNumber int) {
	if partNumber-1 > len(p.Parts) {
		panic("cannot set part number to inexistent partition")
	}

	p.biosGrub = partNumber
}

// SetBoot tells Parted that a partition number is to be the active boot
// partition.
func (p *Parted) SetBoot(partNumber int) {
	if partNumber-1 > len(p.Parts) {
		panic("cannot set part number to inexistent partition")
	}

	p.bootPartition = partNumber
}

func mib2Blocks(size int64) int64 {
	s := size * 1024 * 1024 / 512

	if s%4 != 0 {
		panic(fmt.Sprintf("invalid partition size: %d", s))
	}

	return s
}
