package parted

import (
	"errors"
	"fmt"
	"os/exec"
	"strings"

	"launchpad.net/snappy/tools/fs"
)

// Partition is a partition withing an image or disk.
type Partition struct {
	begin int64
	end   int64
	fs    FsType
	Label PartitionLabel
	Node  string
}

// Format formats the selected partition
func (p *Partition) Format() error {
	var cmd []string

	switch p.fs {
	case FsFat32:
		cmd = []string{"mkfs.vfat", "-F", "32", "-n", string(p.Label)}

		size, err := sectorSize(p.Node)
		if err != nil {
			return err
		}

		if size != "512" {
			cmd = append(cmd, "-s", "1")
		}

		cmd = append(cmd, "-S", size, p.Node)
	case FsExt4:
		cmd = []string{"mkfs.ext4", "-F", "-L", string(p.Label), p.Node}
	default:
		return errors.New("unsupported format")
	}

	return execute(cmd)
}

// Mount mounts the partition to mountpath
func (p *Partition) Mount(mountpath string) error {
	return fs.Mount(p.Node, mountpath)
}

// Unmount umounts the partition
func (p *Partition) Unmount() error {
	return execute([]string{"umount", p.Node})
}

func sectorSize(dev string) (string, error) {
	out, err := exec.Command("blockdev", "--getss", dev).CombinedOutput()
	if err != nil {
		return "", fmt.Errorf("unable to determine block size: %s", out)
	}

	return strings.TrimSpace(string(out)), err
}

// FindPartition finds the partiton with label and returns it's index or error
// if not found
func FindPartition(p []Partition, label PartitionLabel) (int, error) {
	for i := range p {
		if p[i].Label == label {
			return i, nil
		}
	}

	return 0, errors.New("partition does not exist")
}

// PartitionExists returns true if the label is found in p
func PartitionExists(p []Partition, label PartitionLabel) bool {
	_, err := FindPartition(p, label)

	return err == nil
}
